package org.jbpm.examples;

import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.SystemEventListenerFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.ResourceFactory;
import org.drools.logger.KnowledgeRuntimeLogger;
import org.drools.logger.KnowledgeRuntimeLoggerFactory;
import org.drools.persistence.jpa.JPAKnowledgeService;
import org.drools.runtime.Environment;
import org.drools.runtime.EnvironmentName;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.process.ProcessInstance;
import org.jbpm.process.workitem.wsht.CommandBasedWSThroughJMSHumanTaskHandler;
import org.jbpm.task.Group;
import org.jbpm.task.User;
import org.jbpm.task.service.TaskClient;
import org.jbpm.task.service.TaskServer;
import org.jbpm.task.service.TaskService;
import org.jbpm.task.service.TaskServiceSession;
import org.jbpm.task.service.jms.JMSTaskClientConnector;
import org.jbpm.task.service.jms.JMSTaskClientHandler;
import org.jbpm.task.service.jms.JMSTaskServer;

import bitronix.tm.resource.jdbc.PoolingDataSource;


public class JMSTaskServerTest 
{
	
	public static final void main(String[] args) {
		try {
			//1. My QueueConnectionFactory is defined within an application server
			Hashtable<String, String> props = new Hashtable<String, String>();
			props.put("java.naming.factory.initial", "com.sun.enterprise.naming.SerialInitContextFactory");
			props.put("java.naming.factory.url.pkgs", "com.sun.enterprise.naming");
			props.put("java.naming.factory.state", "com.sun.corba.ee.impl.presentation.rmi.JNDIStateFactoryImpl");
			props.put("java.naming.provider.url", "localhost:1099");
			Context context = new InitialContext(props);
			
			//2. Initialize JMS Task service
			Map<String, String> props1 = new HashMap<String, String>();
			props1.put("hibernate.dialect", "org.hibernate.dialect.MySQLDialect");
			props1.put("hibernate.connection.driver_class", "com.mysql.jdbc.Driver");
			props1.put("hibernate.connection.url", "jdbc:mysql://localhost:3306/jbpm");
			props1.put("hibernate.connection.username", "root");
			props1.put("hibernate.connection.password", "root");			
			EntityManagerFactory emf1 = Persistence.createEntityManagerFactory("org.jbpm.task", props1);
			TaskService localTaskService = new TaskService(emf1, SystemEventListenerFactory.getSystemEventListener());
			TaskServiceSession localTaskServiceSession = localTaskService.createSession();			
			for (int i = 0; i < 10; i++) {
				User user = new User("usr" + i);
				localTaskServiceSession.addUser(user);
			}
			localTaskServiceSession.addUser(new User("krisv"));
			for (int j = 0; j < 3; j++) {
				localTaskServiceSession.addGroup(new Group("grp" + j));
			}
			Properties serverProperties = new Properties();
			serverProperties.setProperty("JMSTaskServer.connectionFactory", "ConnectionFactory");
			serverProperties.setProperty("JMSTaskServer.transacted", "true");
			serverProperties.setProperty("JMSTaskServer.acknowledgeMode", "AUTO_ACKNOWLEDGE");
			serverProperties.setProperty("JMSTaskServer.queueName", "tasksQueue");
			serverProperties.setProperty("JMSTaskServer.responseQueueName", "tasksResponseQueue");			
			
			TaskServer server = new JMSTaskServer(localTaskService, serverProperties, context);
			Thread thread = new Thread(server);
			thread.start();
			
			System.out.println("Waiting for the JMS Task Server to come up");
	        while (!server.isRunning()) {
	            System.out.print(".");
	            Thread.sleep(50);
	        }
			
			Properties clientProperties = new Properties();
			clientProperties.setProperty("JMSTaskClient.connectionFactory", "ConnectionFactory");
			clientProperties.setProperty("JMSTaskClient.transactedQueue", "true");
			clientProperties.setProperty("JMSTaskClient.acknowledgeMode", "AUTO_ACKNOWLEDGE");
			clientProperties.setProperty("JMSTaskClient.queueName", "tasksQueue");
			clientProperties.setProperty("JMSTaskClient.responseQueueName", "tasksResponseQueue");			
			TaskClient client = new TaskClient(
					new JMSTaskClientConnector(
							"org.jbpm.process.workitem.wsht.WSThroughJMSHumanTaskHandler",
							new JMSTaskClientHandler(SystemEventListenerFactory.getSystemEventListener()),
							clientProperties, context
					)
			);
			client.connect();
			
			//3.1. Initialize persistent knowledge: JTA data source managed by Bitronix 
			Class.forName("org.gjt.mm.mysql.Driver");			
			PoolingDataSource ds = new PoolingDataSource();
			ds.setUniqueName("jdbc/processInstanceDS");
			ds.setClassName("com.mysql.jdbc.jdbc2.optional.MysqlXADataSource");			
			ds.setMaxPoolSize(3);
			ds.setAllowLocalTransactions(true);
			ds.getDriverProperties().put("user", "root");
			ds.getDriverProperties().put("password", "root");
			ds.getDriverProperties().put("URL", "jdbc:mysql://localhost:3306/jbpm");
			ds.init();
			
			//3.2. Initialize persistent knowledge: Init. persistent session
			EntityManagerFactory emf = Persistence.createEntityManagerFactory("org.jbpm.persistence.jpa");			
			KnowledgeBase kbase = readKnowledgeBase();								
			Environment env = KnowledgeBaseFactory.newEnvironment();
			env.set( EnvironmentName.ENTITY_MANAGER_FACTORY, emf );
			StatefulKnowledgeSession ksession = JPAKnowledgeService.newStatefulKnowledgeSession( kbase, null, env );			
			KnowledgeRuntimeLogger logger = KnowledgeRuntimeLoggerFactory.newThreadedFileLogger(ksession, "test", 1000);
			
			//3.3. Register process session to local task service session			
			ksession.getWorkItemManager().registerWorkItemHandler("Human Task", new CommandBasedWSThroughJMSHumanTaskHandler(ksession));
			
			
			//4. Start a new process instance
			Map<String, Object> params = new HashMap<String, Object>();
			params.put("employee", "krisv");
			params.put("reason", "Yearly performance evaluation");			
			ProcessInstance process = ksession.startProcess("com.sample.evaluation", params);
			System.out.println("State: " + process.getState());
			System.out.println((process == null) ? null : process.getId());
			
			//5. Free resources
			ksession.dispose();
			logger.close();
			
		} catch (Throwable t) {
			t.printStackTrace();
		}
	}

	private static KnowledgeBase readKnowledgeBase() throws Exception {		
		KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
		kbuilder.add(ResourceFactory.newClassPathResource("evaluation/Evaluation.bpmn"), ResourceType.BPMN2);
		return kbuilder.newKnowledgeBase();
	}
}